@@ -398,8 +398,10 @@ static void dterm_bind_reply (void)
 		/* nothing to send */
 		cnt = bufptr;		/* no data left in this packet */
 		read_present = TRUE;	/* always a "read present" */
+		break;
 	case CTERM:
 		/* keep the compiler happy by mentioning this case */
+		break;
 	}
 }
 /*-------------------------------------------------------------------------*/
@@ -442,7 +444,7 @@ static void	ct_setup_link(void)
 
 
 	// Try very hard to get the local username
-    	local_user = getlogin();
+    	local_user = cuserid(NULL);
     	if (!local_user || local_user == (char *)0xffffffff)
       		local_user = getenv("LOGNAME");
     	if (!local_user) local_user = getenv("USER");
@@ -1506,7 +1508,6 @@ static void ct_checkinput_req(void)
 /*-------------------------------------------------------------------------*/
 static void ct_read_pkt(void)
 {
-	struct	timeval	tv;
 	fd_set	rdfs;
 	int	retval;
 	long	numbytes;
@@ -1514,9 +1515,7 @@ static void ct_read_pkt(void)
 	do {
 		FD_ZERO(&rdfs);
 		FD_SET(sockfd,&rdfs);
-		tv.tv_sec = 0;
-		tv.tv_usec= 200;
-		retval=select(sockfd+1,&rdfs,NULL,NULL,&tv); 
+		retval=select(sockfd+1,&rdfs,NULL,NULL, NULL); 
 	} while (retval <= 0); 
 
 	ioctl(sockfd,FIONREAD,&numbytes);
@@ -1643,6 +1642,7 @@ static void ct_proc_pkt(void)
 			break;
 		case VMS:
 			/* keep the compiler happy by mentioning this case */
+			break;
 		}
 		if (cnt==bufptr) ct_read_pkt();
 	}
@@ -1724,6 +1724,7 @@ int main(int argc, char *argv[])
 	    break;
     case VMS:
 	    /* keep the compiler happy by mentioning this case */
+	    break;
     }
     sa.sa_mask  = ss;
     sa.sa_flags = 0;
