@@ -1,5 +1,5 @@
 /******************************************************************************
-    (c) 1998-2000 P.J. Caulfield               patrick@pandh.demon.co.uk
+    (c) 1998-2000 P.J. Caulfield               patrick@tykepenguin.cix.co.uk
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -253,16 +253,16 @@ bool fal_open::process_message(dap_message *m)
 	    switch (am->get_cmpfunc())
 	    {
 	    case dap_accomp_message::END_OF_STREAM:
-
-		fclose(stream);
-
+		
 		// write metafile
 		if (params.use_metafiles && create)
 		    create_metafile(gl.gl_pathv[glob_entry], attrib_msg);
 
 		// move onto next file
-		if (++glob_entry < gl.gl_pathc) // gl.gl_pathv[++glob_entry])
+		if (glob_entry < gl.gl_pathc-1)
 		{
+		  glob_entry++;
+		  fclose(stream);
 		  stream = fopen(gl.gl_pathv[glob_entry], write_access?"w":"r");
 		  if (!stream)
 		  {
@@ -287,11 +287,22 @@ bool fal_open::process_message(dap_message *m)
 
 	    case dap_accomp_message::CLOSE:
 		// finished task
-
+		if (stream)
+		{		    
+		    fclose(stream);
+		    stream = NULL;
+		}
+		
 		// Do close options
-		if (am->get_fop_bit(dap_attrib_message::FB$SPL)) print_file();
-		if (am->get_fop_bit(dap_attrib_message::FB$DLT)) delete_file();
-		if (am->get_fop_bit(dap_attrib_message::FB$TEF)) truncate_file();
+		if (am->get_fop_bit(dap_attrib_message::FB$SPL) ||
+		    attrib_msg->get_fop_bit(dap_attrib_message::FB$SPL))
+		    print_file();
+		if (am->get_fop_bit(dap_attrib_message::FB$DLT) ||
+		    attrib_msg->get_fop_bit(dap_attrib_message::FB$DLT))
+		    delete_file();
+		if (am->get_fop_bit(dap_attrib_message::FB$TEF) ||
+		    attrib_msg->get_fop_bit(dap_attrib_message::FB$TEF))
+		    truncate_file();
 
 		reply.set_cmpfunc(dap_accomp_message::RESPONSE);
 		reply.write(conn);
@@ -349,14 +360,6 @@ bool fal_open::send_file(int rac, long vbn)
     dap_data_message data_msg;
     bool  ateof(false);
 
-    // If we are doing a file transfer than use as big a buffer as
-    // possible (allowing for control headers), but rounded
-    // down to a multiple of 512 bytes to match VMS block sizes
-    if (streaming && !use_records) 
-    {
-	bs = conn.get_blocksize()-10;
-	bs -= bs%512;
-    }
 
     if (verbose > 2) DAPLOG((LOG_DEBUG, "sending file contents. block size is %d. streaming = %d, use_records=%d, vbn=%d\n", bs, streaming, use_records, vbn));
 
@@ -391,56 +394,53 @@ bool fal_open::send_file(int rac, long vbn)
 	{
 	    // Do we need to use the stored record lengths from the metafile ?
 	    if (attrib_msg->get_rfm() == dap_attrib_message::FB$VAR &&
-		record_lengths != NULL && current_record < num_records)
+		record_lengths != NULL)
 	    {
-		if (::fread(buf, 1, record_lengths[current_record], stream) < 1)
-		    ateof = true;
-
-		// We read a whole record (including our "compatibility" LF)
-		// which VMS does not want.
-		buflen = record_lengths[current_record++]-1;
-	    }
-	    else
-	    {
-		if (::fgets(buf, bs-1, stream))
+		if (current_record < num_records)
 		{
-		    // Make sure there's a NUL on the end for strlen to find.
-		    buf[bs] = '\0';
-		    buflen = strlen(buf); // Leave the LF on the end.
-		    if (!buflen) ateof = true;
+		    if (::fread(buf, 1, record_lengths[current_record], stream) < 1)
+			ateof = true;
+		    
+		    // We read a whole record (including our "compatibility" LF)
+		    // which VMS does not want.
+		    buflen = record_lengths[current_record++]-1;
 		}
 		else
 		{
-		    DAPLOG((LOG_INFO, "fgets returned nothing\n"));
 		    send_eof();
 		    return true;
 		}
 	    }
+	    else
+	    {
+		// Read up to the next LF or EOF
+		int newchar;
+		buflen = 0;
+		do 
+		{
+		    newchar = getc(stream);
+		    if (newchar != EOF)
+		    {
+			buf[buflen++] = (char) newchar;
+		    }
+		} while (newchar != EOF && newchar != '\n' && buflen < conn.get_blocksize());
+	    }
 	}
 	else // Block read
 	{
 	    buflen = ::fread(buf, 1, bs, stream);
 	    if (!buflen) ateof = true;
+
+	    // Always send a full block or VMS complains.
+	    buflen=bs;
 	}
 	
 	// We got some data
 	if (!ateof)
 	{
 	    data_msg.set_data(buf, buflen);
+	    if (!data_msg.write_with_len(conn)) return false;
 
-	    // Complicated bit dealing with headers.
-	    // If we are streaming then performance is important so we send
-	    // as much as possible in one block with no length.
-	    // When not streaming we send a length ('cos we need to send a
-	    // STATUS message too) and we use the shortest len header we can.
-	    if (streaming && !use_records)
-            {
-		if (!data_msg.write(conn)) return false;
- 	    }
-	    else
-	    {
-		if (!data_msg.write_with_len(conn)) return false;
-            }
 	    if (verbose > 2) DAPLOG((LOG_DEBUG, "sent %d bytes of data\n", buflen));
 	    total += buflen;
 	}
@@ -472,6 +472,11 @@ bool fal_open::send_file(int rac, long vbn)
 		if (d->get_type() == dap_message::ACCOMP)
 	        {
 		    dap_accomp_message am;
+
+		    // Make sure this is the first thing sent. Anything pending
+		    // is no longer required.
+		    conn.clear_output_buffer();
+
 		    am.set_cmpfunc(dap_accomp_message::RESPONSE);
 		    am.write(conn);
 		    conn.set_blocked(false); 
@@ -552,7 +557,8 @@ bool fal_open::put_record(dap_data_message *dm)
 
     // If we are writing variable-length records then keep a list of
     // their lengths in the metadata.
-    if (attrib_msg->get_rfm() == dap_attrib_message::FB$VAR && use_records)
+    if (attrib_msg->get_rfm() == dap_attrib_message::FB$VAR && use_records &&
+	params.use_metafiles)
     {
 	if (!record_lengths)
 	{
