@@ -1,5 +1,5 @@
 /******************************************************************************
-    (c) 1998-2000 P.J. Caulfield               patrick@pandh.demon.co.uk
+    (c) 1998-2000 P.J. Caulfield               patrick@tykepenguin.cix.co.uk
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -71,7 +71,7 @@ void fal_task::set_crc(bool onoff)
     need_crc = onoff;
 }
 
-void fal_task::calculate_crc(char *buf, int len)
+void fal_task::calculate_crc(unsigned char *buf, int len)
 {
     crc.calc4shift(buf, len);
 }
@@ -390,6 +390,24 @@ void fal_task::make_unix_filespec(char *unixname, char *vmsname)
     // can't really distinguish case. I know samba does fancy stuff with
     // matching various combinations of case but I really can't be bothered.
     dap_connection::makelower(unixname);
+
+    // If the name ends in .dir and there is a directory of that name without
+    // the .dir then remove it (the .dir, not the directory!)
+    if (strstr(unixname, ".dir") == unixname+strlen(unixname)-4)
+    {
+	char dirname[strlen(unixname)+1];
+	struct stat st;
+
+	strcpy(dirname, unixname);
+	char *ext = strstr(dirname, ".dir");
+	if (ext) *ext = '\0';
+	if (stat(dirname, &st) == 0 &&
+	    S_ISDIR(st.st_mode))
+	{
+	    char *ext = strstr(unixname, ".dir");
+	    if (ext) *ext = '\0';
+	}
+    }
 }
 
 // Convert VMS wildcards to Unix wildcards
@@ -598,6 +616,7 @@ bool fal_task::guess_file_type(unsigned int &block_size, bool &send_records,
 	    attrib_msg->clear_rat_bit(dap_attrib_message::FB$CR);
 	    block_size = 512;
 	    if (verbose > 1) DAPLOG((LOG_INFO, "sending file %s as blocked binary\n", name));
+	    send_records = false;
 	    return true;
 	}
 	else
@@ -943,9 +962,11 @@ void fal_task::create_metafile(char *name, dap_attrib_message *attrib_msg)
 	for (unsigned int i=0; i<current_record; i++)
 	    if (record_lengths[i] > lrl) lrl = record_lengths[i];
 	
-	// Our record lengths include the terminating LF
-	metadata.lrl = lrl - 1;
-	
+	// Our record lengths include the terminating LF so 
+	// subtract that from the length.
+        // If there are no records then put MRS in there.
+	metadata.lrl = (lrl?(lrl - 1):metadata.mrs);
+
 	// Write it out.
 	if (::fwrite(&metadata, sizeof(metadata), 1, mf) != 1)
 	{
@@ -1053,18 +1074,21 @@ fal_task::auto_types *fal_task::auto_types_list = NULL;
 const char *fal_task::sysdisk_name = "SYSDISK";
 
 const char *fal_task::default_types_file  = "#\n\
-#Generic types
+#Generic types\n\
 .txt  r\n\
 .c    r\n\
 .cc   r\n\
 .log  r\n\
-# VMS types
+.html r\n\
+# VMS types\n\
 .com  r\n\
 .lis  r\n\
 .bck  b 32256\n\
 .save b 8192\n\
 .exe  b 512\n\
-#Linux types
+#Linux types\n\
 .tar  b 10240\n\
 .gz   b 512\n\
+.tgz  b 512\n\
+.bz2  b 512\n\
 # End of file\n";
