@@ -1,7 +1,7 @@
 /*
     protocol.cc from libdap
 
-    Copyright (C) 1998-1999 Patrick Caulfield       patrick@pandh.demon.co.uk
+    Copyright (C) 1998-2001 Patrick Caulfield       patrick@tykepenguin.cix.co.uk
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -36,6 +36,7 @@
 #include <errno.h>
 #include <regex.h>
 #include <ctype.h>
+#include <time.h>
 #include <netdnet/dn.h>
 #include <netdnet/dnetdb.h>
 
@@ -120,6 +121,12 @@ void dap_bytes::set_string(const char *newval)
     length = strlen(newval);
     strcpy((char *)value, newval);
 }
+
+void dap_bytes::set_value(const char *newval, int len)
+{
+    length = len;
+    memcpy((char *)value, newval, len);
+}
 //---------------------------------- dap_ex() ------------------------------
 bool dap_ex::read(dap_connection &c)
 {
@@ -213,7 +220,6 @@ bool dap_image::read(dap_connection &c)
 {
     char *b = c.getbytes(1);
     if (!b) return false;
-
     real_length = *b;
 
     b = c.getbytes(real_length);
@@ -279,6 +285,12 @@ void dap_image::set_string(const char *s)
     real_length = strlen(s);
 }
 
+void dap_image::set_value(const char *newval, int len)
+{
+    real_length = len;
+    memcpy((char *)value, newval, len);
+}
+
 //-----------------------------------------------------------------------------
 //-------------------------- DAP Messages start here --------------------------
 //-----------------------------------------------------------------------------
@@ -333,6 +345,8 @@ int dap_message::send_long_header(dap_connection &c)
 // the message length (used mainly for data messages)
 int dap_message::get_header(dap_connection &c)
 {
+    c.check_length(1); // Ensure we have the start of a header
+
     char *b = c.getbytes(1);
     if (!b) return false;
 
@@ -379,14 +393,12 @@ int dap_message::get_header(dap_connection &c)
 	c.getbytes(1);
     }
 
-    if (flags & 16) // SYSPEC is hard.
+    if (flags & 32) // SYSPEC is impossible
     {
-	fprintf(stderr, "SYSPEC field not supported yet.\n
-                         Please email patrick@pandh.demon.co.uk if you REALLY want it.\n");
+	DAPLOG((LOG_WARNING, "got SYSPEC field - aborting\n"));
 	exit(999);
     }
 
-    // TODO SYSPEC
     return true;
 }
 
@@ -395,15 +407,11 @@ int dap_message::get_header(dap_connection &c)
 // If no message is available then -1 is returned.
 int dap_message::peek_message_type(dap_connection& c)
 {
-//    if (!c.read_if_necessary(false))
-//	return -1;
-    
     char *b = c.peekbytes(1);
     if (b)
 	return *b;
     else
-	return -1;
-    
+	return -1;    
 }
 
 // Read a message from the stream and spit out a fully formed message
@@ -957,6 +965,12 @@ void dap_control_message::set_key(const char *k)
     ctlmenu.set_bit(1);
 }
 
+void dap_control_message::set_key(const char *k, int l)
+{
+    key.set_value(k, l);
+    ctlmenu.set_bit(1);
+}
+
 void dap_control_message::set_krf(int f)
 {
     krf.set_byte(0,f);
@@ -1621,6 +1635,12 @@ bool dap_date_message::read(dap_connection &c)
     if (datmenu.get_bit(2) && !edt.read(c)) return false;
     if (datmenu.get_bit(3) && !rvn.read(c)) return false;
     if (datmenu.get_bit(4) && !bdt.read(c)) return false;
+
+// Bit 6 is the Ultrix changed date but I don't know what bit 5 is.
+// Since we don't use either date I'll just dump them both in the same 
+// place for now (and hope they are both in the same format!)
+    if (datmenu.get_bit(5) && !udt.read(c)) return false;
+    if (datmenu.get_bit(6) && !udt.read(c)) return false;
     return true;
 }
 
@@ -1644,6 +1664,11 @@ char *dap_date_message::get_bdt()
     return bdt.get_string();
 }
 
+char *dap_date_message::get_udt()
+{
+    return bdt.get_string();
+}
+
 time_t dap_date_message::get_cdt_time()
 {
     return string_to_time_t(edt.get_string());
@@ -1659,12 +1684,16 @@ time_t dap_date_message::get_edt_time()
     return string_to_time_t(edt.get_string());
 }
 
-
 time_t dap_date_message::get_bdt_time()
 {
     return string_to_time_t(bdt.get_string());
 }
 
+time_t dap_date_message::get_udt_time()
+{
+    return string_to_time_t(bdt.get_string());
+}
+
 int dap_date_message::get_rvn()
 {
     return rvn.get_short();
@@ -1704,6 +1733,8 @@ char *dap_date_message::time_to_string(time_t t)
     static char d[25];
     struct tm tm = *localtime(&t);
 
+// This causes a warning because of the 2-digit year but
+// it's what DAP requires!
     strftime(d, sizeof(d), "%d-%b-%y %H:%M:%S", &tm);
 
 // Convert the month to uppercase (strftime makes the last two letters lower)
@@ -1718,7 +1749,7 @@ time_t dap_date_message::string_to_time_t(const char *d)
     struct tm tm;
     char month[5];
 
-    sscanf(d, "%d-%s-%d %d:%d:%d", 
+    sscanf(d, "%d-%3s-%d %d:%d:%d", 
 	   &tm.tm_mday, month,      &tm.tm_year, 
 	   &tm.tm_hour, &tm.tm_min, &tm.tm_sec);
 
@@ -1741,20 +1772,43 @@ char *dap_date_message::make_y2k(char *dt)
 {
     static char y2kdate[25];
     int year;
+    int timepos;
 
-    strcpy(y2kdate, dt);
+    strcpy(y2kdate, dt);   
     
-    year = (dt[7]-'0')*10 + dt[8]-'0';
-    if (year >= 70) 
-	year += 1900;
+// Workaround for Y2K bug in Ultrix:
+// Dates come back as three digits with 2000 as 100
+    if (dt[9] == ' ') 
+    {
+	// Correct behaviour
+	year = (dt[7]-'0')*10 + dt[8]-'0';
+	if (year >= 70) 
+	    year += 1900;
+	else
+	    year += 2000;
+	timepos = 7;
+    }
     else
-	year += 2000;
+    {
+	// Broken behaviour
+	year = (dt[7]-'0')*100 + (dt[8]-'0')*10 + dt[9]-'0';
+	if (year >= 70) 
+	    year += 1900;
+	else
+	    year += 2000;
+	timepos = 8;
+    }
 
     char yearstr[5];
     sprintf(yearstr, "%04d", year);
     y2kdate[7] = '\0';
     strcat(y2kdate, yearstr);
-    strcat(y2kdate, dt+9);
+    strcat(y2kdate, dt+timepos+2);
+
+    // Because the year is 3 digits on broken Ultrix, the seconds
+    // are truncated to 1 digit. Add a trailing zero so they look nice.
+    if (timepos == 8)
+	strcat(y2kdate, "0");
 
     return y2kdate;
 }
@@ -2046,9 +2100,6 @@ bool dap_key_message::read(dap_connection &c)
 	    siz[i] = new dap_bytes(1);
 	    pos[i]->read(c);
 	    siz[i]->read(c);
-
-	    fprintf(stderr, "key %d, pos= %d, size=%d\n",
-		    i, pos[i]->get_int(),siz[i]->get_int());
 	}
     }
     if (keymenu.get_bit(4) && !ref.read(c)) return false;
